=begin header
    Gimp-Ruby - allows the writing of Gimp plugins in Ruby.
    Copyright (C) 2001  Masahiro Sakai
 
          Masahiro Sakai                  <zvm01052@nifty.ne.jp>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
=end

require 'gimp.so'
require 'gimpenums'

module Gimp

###############################################################################

  def method_missing(msg_id, *args)
    proc_name = "gimp_" + msg_id.id2name
    if PDB.IsProcedure?(proc_name)
      *vals = PDB.Call(proc_name, *args)
      case vals.size
      when 0
        Gimp
      when 1
        vals[0]
      else
        vals
      end
    else
      super
    end
  end
  module_function :method_missing

###############################################################################
# PDB Utilities

  module PDB
    def Call(proc_name, *args)
      run_procedure(proc_name, *args)
    end
    module_function :Call

    def method_missing(msg_id, *args)
      Call(msg_id.id2name, *args)
    end
    module_function :method_missing

    ## Exception

    class ErrorStatus < RuntimeError #abstract class
      private_class_method :new
    end


    #

    Procedure = Struct.new("Procedure",
                           :name, :blurb, :help, :author, :copyright,
                           :date, :proc_type, :args, :values)
    class <<Procedure
      def call(*args)
        PDB.Call(self.name, *args)
      end
    end

    def ProcInfo(proc_name)
      begin
        blurb, help, author, copyright, date, proc_type,
          num_args, num_vals = gimp_procedural_db_proc_info(proc_name)
        args = (0...num_args).collect { |i|
          gimp_procedural_db_proc_arg(proc_name, i)
        }
        vals = (0...num_vals).collect { |i|
          gimp_procedural_db_proc_val(proc_name, i)
        }
        return Procedure.new(proc_name, blurb, help, author, copyright, date,
                             proc_type, args, vals)
      rescue ExecutionError
        return nil
      end
    end
    module_function :ProcInfo


    def IsProcedure?(proc_name)
      begin
        gimp_procedural_db_proc_info(proc_name)
        return true
      rescue ExecutionError
        return false;
      end
    end
    module_function :IsProcedure?

  end

###############################################################################
# Color

  class Color
    def ==(rhs)
      (rhs.is_a? Color) and (self.red == rhs.red) and
        (self.green == rhs.green) and (self.blue == rhs.blue)
    end

    def -@
      self.red   = 255 - self.red
      self.green = 255 - self.green
      self.blue  = 255 - self.blue
    end

    def clamp
      if (red > 255)
        red = 255
      elsif red < 0
        red = 0
      end

      if (green > 255)
        green = 255
      elsif green < 0
        green = 0
      end

      if (blue > 255)
        blue = 255
      elsif blue < 0
        blue = 0
      end
    end
 
    def +(rhs)
      case rhs
      when Color
        Color::new(self.red + rhs.red,
                   self.green + rhs.green,
                   self.blue + rhs.blue).clamp
      else
        coerced_me, coerced_other = rhs.coerce(self)
        coerced_me + coerced_other
      end
    end

    def -(rhs)
      case rhs
      when Color
        Color::new(self.red - rhs.red,
                   self.green - rhs.green,
                   self.blue - rhs.blue).clamp
      else
        coerced_me, coerced_other = rhs.coerce(self)
        coerced_me - coerced_other
      end
    end

    def *(rhs)
      case rhs
      when Color
        Color::new(self.red * rhs.red / 255,
                   self.green * rhs.green / 255,
                   self.blue * rhs.blue / 255)
      else
        coerced_me, coerced_other = rhs.coerce(self)
        coerced_me * coerced_other
      end
    end

    def /(rhs)
      case rhs
      when Color
        Color::new(self.red * 255 / rhs.red,
                   self.green * 255 / rhs.green,
                   self.blue * 255 / rhs.blue)
      else
        coerced_me, coerced_other = rhs.coerce(self)
        coerced_me * coerced_other
      end
    end

    def coerce(other)
      raise TypeError, "Unknown type"
    end

    def eql?(rhs)
      self == rhs
    end

    def hash
      ((self.red << 24) | (self.green << 16) | (self.blue << 8)).hash
    end

    def _dump(limit)
      Array(self).pack("CCC")
    end

    def Color._load(str)
      Color.new(*(str.unpack("CCC")))
    end
  end

###############################################################################
# Region

  class Region
    def ==(rhs)
      (rhs.is_a? Region) and (self.x == rhs.x) and (self.y == rhs.y) and
        (self.width == rhs.width) and (self.height == rhs.height)
    end

    def eql?(rhs)
      self == rhs
    end

    def hash
      (x + y + width + height).hash
    end

    def _dump(limit)
      Array(self).pack("llll")
    end

    def Region._load(str)
      Region.new(*(str.unpack("llll")))
    end
  end

###############################################################################
# ObjectID
  class ObjectID
    def CheckSanity
      raise "the object is already deleted" unless @gimp_id
    end
    
    def to_i
      @gimp_id
    end
  end

###############################################################################
# Image

  class Image < ObjectID
    def delete
      CheckSanity()
      begin
        PDB.gimp_image_delete(self)
      ensure
        @gimp_id = nil
      end
    end

    def get_layers
      num, layers = PDB.gimp_image_get_layers(self)
      layers.collect!{|item| Drawable::new_from_id(item)}
    end

    def get_channels
      num, channels = PDB.gimp_image_get_channels(self)
      channels.collect!{|item| Drawable::new_from_id(item)}
    end

    # FIXME:
    # $BESCf$G(Bgimp-image-delete-guide$B$5$l$k$H$3$N%$%F%l!<%?$O$&$^$/F0:n$7$J$$(B!
    def each_guide
      i = 0
      begin
        i = find_next_guide(self, i)
        yield i
      end until i == 0
    end

    def method_missing(msg_id, *args)
      CheckSanity()
      proc_name = "gimp_image_" + msg_id.id2name
      if PDB.IsProcedure?(proc_name)
        *vals = PDB.Call(proc_name, self, *args)
        case vals.size
        when 0
          self
        when 1
          vals[0]
        else
          vals
        end
      else
        super
      end
    end

    def Image.list
      num, list = PDB.gimp_image_list()
      list.collect!{|item| Image::new_from_id(item) }
    end

  end

###############################################################################
# Drawable

  class Drawable < ObjectID
    def method_missing(msg_id, *args)
      CheckSanity()
      proc_name = "gimp_drawable_" + msg_id.id2name
      if PDB.IsProcedure?(proc_name)
        *vals = PDB.Call(proc_name, self, *args)
        case vals.size
        when 0
          self
        when 1
          vals[0]
        else
          vals
        end
      else
        super
      end
    end
  end

###############################################################################
# Layer

  class Layer < Drawable
    def delete
      CheckSanity()
      begin
        PDB.gimp_layer_delete(self)
      ensure
        @gimp_id = nil
      end
    end

    def method_missing(msg_id, *args)
      CheckSanity()
      method_name = msg_id.id2name
      proc_name = nil

      if PDB.Call("gimp_layer_is_floating_sel", self) != 0
        tmp = "gimp_floating_sel_" + method_name
        if PDB.IsProcedure?(tmp)
          proc_name = tmp
        end
      end

      unless proc_name
        tmp = "gimp_layer_" + method_name
        if PDB.IsProcedure?(tmp)
          proc_name = tmp
        end
      end

      unless proc_name
        super
      else
        *vals = PDB.Call(proc_name, self, *args)
        case vals.size
        when 0
          self
        when 1
          vals[0]
        else
          vals
        end
      end
    end
  end

###############################################################################
# Channel

  class Channel < Drawable
    def delete
      CheckSanity()
      begin
        PDB.gimp_channel_delete(self)
      ensure
        @gimp_id = nil
      end
    end

    def method_missing(msg_id, *args)
      CheckSanity()
      proc_name = "gimp_channel_" + msg_id.id2name
      if PDB.IsProcedure?(proc_name)
        *vals = PDB.Call(proc_name, self, *args)
        case vals.size
        when 0
          self
        when 1
          vals[0]
        else
          vals
        end
      else
        super
      end
    end
  end

###############################################################################
# Display

  class Display < ObjectID
    def delete
      CheckSanity()
      begin
        PDB.gimp_display_delete(self)
      ensure
        @gimp_id = nil
      end
    end

    def method_missing(msg_id, *args)
      CheckSanity()
      proc_name = "gimp_display_" + msg_id.id2name
      if PDB.IsProcedure?(proc_name)
        *vals = PDB.Call(proc_name, self, *args)
        case vals.size
        when 0
          self
        when 1
          vals[0]
        else
          vals
        end
      else
        super
      end
    end

  end

###############################################################################
# Unit

  if defined? Unit

    class Unit < ObjectID
      extend ::Enumerable

      def Unit.number_of_units
        PDB.gimp_unit_get_number_of_units
      end

      def Unit.number_of_built_in_units
        PDB.gimp_unit_get_number_of_built_in_units
      end

      def Unit.each
        for i in 0..number_of_units()
          yield Unit::new_from_id(i)
        end
      end

      def Unit.each_built_in
        for i in 0..number_of_units_built_in()
          yield Unit::new_from_id(i)
        end
      end
    end

  end

###############################################################################
# CDrawable

  class CDrawable
    def each_tile(shadow = false)
      for row in 0..(ntile_rows-1)
        for col in 0..(ntile_cols-1)
          yield(get_tile(shadow, row, col))
        end
      end
    end
  end

###############################################################################
# gimpcolorspace.h

  def intensity(r, g, b)
    r * INTENSITY_RED + g * INTENSITY_GREEN + b * INTENSITY_BLUE + 0.001
  end
  module_function :intensity

###############################################################################
# gimpundo.h

  class Image
    def undo_group()
      PDB.gimp_undo_push_group_start(self)
      begin
        yield self
      ensure
        PDB.gimp_undo_push_group_end(self)
      end
    end
  end

###############################################################################
# Message

  module Message
    def handler
      PDB.gimp_message_get_handler
    end

    def handler=(val)
      PDB.gimp_message_set_handler(val)
      val
    end

    def <<(val)
      Gimp::message(val)
      self
    end
  end

###############################################################################
# gimprc

  module RC
    def [](token)
      begin
        PDB.gimp_gimprc_query(token)
      rescue PDB::ExecutionError
        nil
      end
    end

    def []=(token, value)
      PDB.gimp_gimprc_set(token, value)
      value
    end
  end

end
