#!/usr/bin/env ruby

=begin header
COOL-METAL
Create a text effect that looks like metal with a reflection of
the horizon, a reflection of the text in the mirrored ground, and
an interesting dropshadow
This script was inspired by Rob Malda's 'coolmetal.gif' graphic
'Translated into Ruby by Masahiro Sakai <zvm01052@nifty.ne.jp>, 26th Feb 2001
=end

require "gimp"
require "gimpfu"
include Gimp

def apply_cool_metal_logo_effect(img, logo_layer, size, bg_color, gradient)

  feather    = size / 5
  smear      = 7.5
  period     = size / 3
  amplitude  = size / 40
  shrink     = size / 30 + 1
  depth      = size / 20
  width      = logo_layer.width
  height     = logo_layer.height
  img_width  = width + 0.15 * height + 10
  img_height = 1.85 * height + 10
  bg_layer = Layer::new(img, img_width, img_height, RGB_IMAGE, "Background")
  shadow_layer = Layer::new(img, img_width, img_height, RGBA_IMAGE, "Shadow")
  reflect_layer = Layer::new(img, img_width, img_height, RGBA_IMAGE, "Reflection")

  old_gradient = gradients_get_active
  old_fg       = palette_get_foreground
  old_bg       = palette_get_background
  begin
    img.resize(img_width, img_height, 0, 0)
    img.add_layer(bg_layer, 1)
    img.add_layer(reflect_layer, 1)
    img.add_layer(shadow_layer, 1)
    logo_layer.set_preserve_trans(true)

    palette_set_background(bg_color)
    edit_fill(bg_layer, BG_IMAGE_FILL)
    edit_clear(reflect_layer)
    palette_set_background(Color::new(0, 0, 0))
    edit_fill(shadow_layer, BG_IMAGE_FILL)

    gradients_set_active(gradient)
    blend(logo_layer, CUSTOM, NORMAL_MODE, LINEAR, 100, 0, REPEAT_NONE,
	  false, 0, 0, 0, 0, 0, height + 5)
    rect_select(img, 0, height / 2 - feather, img_width, feather * 2,
		REPLACE, 0, 0)
    PDB.plug_in_gauss_iir(RUN_NONINTERACTIVE,
			  img, logo_layer, smear, true, true)
    selection_none(img)
    PDB.plug_in_ripple(RUN_NONINTERACTIVE, img, logo_layer,
		       period, amplitude, 1, 0, 1, true, false)
    logo_layer.translate(5, 5)
    logo_layer.resize(img_width, img_height, 5, 5)
  
    selection_layer_alpha(logo_layer)
    channel = selection_save(img)
    selection_shrink(img, shrink)
    selection_invert(img)
    PDB.plug_in_gauss_rle(RUN_NONINTERACTIVE,
			  img, channel, feather, true, true)
    selection_layer_alpha(logo_layer)
    selection_invert(img)
    palette_set_background(Color::new(0, 0, 0))
    edit_fill(channel, BG_IMAGE_FILL)
    selection_none(img)
  
    PDB.plug_in_bump_map(RUN_NONINTERACTIVE, img, logo_layer, channel,
			 135, 45, depth, 0, 0, 0, 0, false, false, 0)

    selection_layer_alpha(logo_layer)
    fs = selection_float(shadow_layer, 0, 0)
    edit_clear(shadow_layer)
    perspective(fs, false, 5 + 0.15 * height, height - 0.15 * height,
		5 + width + 0.15 * height, height - 0.15 * height,
		5.0, Float(height), 5 + width, Float(height))
    fs.anchor
    PDB.plug_in_gauss_rle(RUN_NONINTERACTIVE,
			  img, shadow_layer, smear, true, true)

    rect_select(img, 5, 5, width, height, REPLACE, false, 0)
    edit_copy(logo_layer)
    fs = edit_paste(reflect_layer, false)
    floating_sel_anchor(fs)
    scale(reflect_layer, false, 0, 0, width, height * 0.85)
    flip(reflect_layer, 1)
    reflect_layer.set_offsets(5, 3 + height)
  
    layer_mask = reflect_layer.create_mask(WHITE_MASK)
    img.add_layer_mask(reflect_layer, layer_mask)
    palette_set_foreground(Color::new(255, 255, 255))
    palette_set_background(Color::new(0, 0, 0))
    blend(layer_mask, FG_BG_RGB, NORMAL_MODE, LINEAR, 100, 0, REPEAT_NONE,
	  false, 0, 0, 0, - height / 2, 0, height)

    img.remove_channel(channel)
  ensure
    gradients_set_active(old_gradient)
    palette_set_background(old_bg)
    palette_set_foreground(old_fg)
  end
end


def cool_metal_logo_alpha(img, logo_layer, size, bg_color, gradient)
  img.undo_group {
    apply_cool_metal_logo_effect(img, logo_layer, size, bg_color, gradient)
  }
  displays_flush
end

register("ruby_fu_cool_metal_logo_alpha",
	 "Metallic logos with reflections and perspective shadows",
	 "Metallic logos with reflections and perspective shadows",
	 "Spencer Kimball & Rob Malda",
	 "Spencer Kimball & Rob Malda",
	 "1997",
	 "<Image>/Ruby-Fu/Alpha to Logo/Cool Metal...",
	 "RGBA",
	 [
	   [RF_IMAGE,      "image",       "Image"],
	   [RF_DRAWABLE,   "drawable",    "Drawable"],
	   [RF_ADJUSTMENT, "effect_size", "Effect Size (pixels)", 100,
	     [2, 1000, 1, 10, 0, 1]],
	   [RF_COLOR,      "bg_color", "Background Color",
	     Color::new(255, 255, 255)],
	   [RF_GRADIENT,   "gradient",  "Gradient", "Horizon_1"]
	 ],
	 []
) { |*args|
  cool_metal_logo_alpha(*args)
}


def cool_metal_logo(text, size, font, bg_color, gradient)
  img = Gimp::Image::new(256, 256, RGB)
  begin
    text_layer = text_fontname(img, -1, 0, 0, text, 0, true, size, PIXELS,
			       font)
    img.undo_disable
    text_layer.set_name(text)
    apply_cool_metal_logo_effect(img, text_layer, size, bg_color, gradient)
    img.undo_enable
  rescue
    img.delete
    raise
  end
  Gimp::Display::new(img)
end

register("ruby_fu_cool_metal_logo",
	 "Metallic logos with reflections and perspective shadows",
	 "Metallic logos with reflections and perspective shadows",
	 "Spencer Kimball & Rob Malda",
	 "Spencer Kimball & Rob Malda",
	 "1997",
	 "<Toolbox>/Xtns/Ruby-Fu/Logos/Cool Metal...",
	 "",
	 [
	   [RF_STRING,     "text",      "Text", "Cool Metal"],
	   [RF_ADJUSTMENT, "font_size", "Font Size (pixels)", 100,
	     [2, 1000, 1, 10, 0, 1]],
	   [RF_FONT,       "font",      "Font",
	     "-*-Crillee-*-r-*-*-24-*-*-*-p-*-*-*"],
	   [RF_COLOR,      "bg_color",  "Background Color",
	     Color::new(255, 255, 255)],
	   [RF_GRADIENT,   "gradient",  "Gradient", "Horizon_1"]
	 ],
	 []
) { |*args|
  cool_metal_logo(*args)
}

main()
