#!/usr/bin/env ruby

=begin
Translated into Ruby by Masahiro Sakai <zvm01052@nifty.ne.jp>, 26th Feb 2001
=end

=begin header
Create a 3D sphere with optional shadow
The sphere's principle color will be the foreground
Parameters:
 bg-color: background color
 sphere-color: color of sphere
 radius: radius of the sphere in pixels
 light:  angle of light source in degrees
 shadow: whather to create a shadow as well
=end

require "gimp"
require "gimpfu"
include Math
include Gimp

def sphere(radius, light, shadow, bg_color, sphere_color)
  width  = radius * 3.75
  height = radius * 2.5

  img = Image::new(width, height, RGB)

  img.undo_disable
  old_fg = palette_get_foreground()
  old_bg = palette_get_background()

  begin
    radians = light * PI / 180
    cx = width / 2
    cy = height / 2
    light_x = cx + radius * 0.6 * cos(radians)
    light_y = cy - radius * 0.6 * sin(radians)
    light_end_x = cx + radius * cos(radians + PI)
    light_end_y = cy - radius * sin(radians + PI)
    offset = radius * 0.1

    drawable = Layer::new(img, width, height, RGB_IMAGE, "Sphere Layer")
    img.add_layer(drawable, 0)

    palette_set_foreground(sphere_color)
    palette_set_background(bg_color)
    edit_fill(drawable, BG_IMAGE_FILL)

    palette_set_background(Color::new(20, 20, 20))

    if shadow and ((45 <= light and light <= 75) or
                   (105 <= light and light <= 135))
      shadow_w = radius * 2.5 * cos(radians + PI)
      shadow_h = radius * 0.5
      shadow_x = cx
      shadow_y = radius * 0.65 + cy
      if shadow_w < 0
        shadow_x = shadow_w + cx
        shadow_w = - shadow_w
      end
      ellipse_select(img, shadow_x, shadow_y, shadow_w, shadow_h,
                     REPLACE, true, true, 7.5)
      bucket_fill(drawable, BG_BUCKET_FILL, MULTIPLY_MODE, 100, 0, false, 0, 0)
    end
    
    ellipse_select(img, cx - radius, cy - radius, radius * 2, radius * 2,
                   REPLACE, true, false, 0)
    blend(drawable, FG_BG_RGB, NORMAL_MODE, RADIAL, 100, offset, REPEAT_NONE,
          FALSE, 0, 0, light_x, light_y, light_end_x, light_end_y)
    selection_none(img)
  ensure
    palette_set_background(old_bg)
    palette_set_foreground(old_fg)
    img.undo_enable
  end

  Display::new(img)
end


register("ruby_fu_sphere",
	 "Simple sphere with a drop shadow",
	 "Simple sphere with a drop shadow", 
	 "Spencer Kimball",
	 "Spencer Kimball",
	 "1996",
	 "<Toolbox>/Xtns/Ruby-Fu/Misc/Sphere...",
	 "",
	 [
	   [RF_ADJUSTMENT, "radius", "Radius (pixels)", 100,
	     [5, 500, 1, 10, 0, 1]],
	   [RF_ADJUSTMENT, "lighting", "Lighting (degrees)", 45,
	     [0, 360, 1, 10, 0, 0]],
	   [RF_TOGGLE, "shadow", "Shadow", true],
	   [RF_COLOR, "bgcolor", "Background Color", Color::new(255,255,255)],
	   [RF_COLOR, "color", "Sphere Color", Color::new(255,0,0)],
	 ]
) {
  |*args|
  sphere(*args)
}


main()
