#!/usr/bin/env ruby

=begin
BLENDED-DROP-SHADOW-LOGO
draw the specified text over a blended background using current gimp fg
 and bg colors.  The finished blend has a drop shadow underneath that blends
 to the specified bg-color
if the blend colors are specified as high intensity, the sharp option
 should be enabled or the logo will come out blurry
=end

require "gimp"
require "gimpfu"
include Gimp


def scale(size, percent)
  size * percent
end


def apply_textured_logo_effect(img, logo_layer,
			       b_size, text_pattern, tile_type,
			       bg_color, blend_fg, blend_bg)
  b_size_2 = scale(b_size, 0.5)
  f_size   = scale(b_size, 0.75)
  ds_size  = scale(b_size, 0.5)
  ts_size  = b_size_2 - 3
  width    = logo_layer.width
  height   = logo_layer.height
  blend_layer  = Layer::new(img, width, height, RGBA_IMAGE, "Blend")
  shadow_layer = Layer::new(img, width, height, RGBA_IMAGE, "Shadow")

  text_shadow_layer = Layer::new(img, "Text Shadow", width, height, RGBA_IMAGE,
				 100, MULTIPLY_MODE)
  tsl_layer_mask = text_shadow_layer.create_mask(BLACK_MASK)

  drop_shadow_layer = Layer::new(img, width, height, RGBA_IMAGE, "Drop Shadow",
				 100, MULTIPLY_MODE)
  dsl_layer_mask = drop_shadow_layer.create_mask(BLACK_MASK)

  old_fg = palette_get_foreground()
  old_bg = palette_get_background()
  old_pattern = patterns_get_pattern

  begin
    img.resize(width, height, 0, 0)
    img.add_layer(shadow_layer, 1)
    img.add_layer(blend_layer, 1)
    img.add_layer(drop_shadow_layer, 1)
    img.add_layer(text_shadow_layer, 0)
    selection_all(img)
    patterns_set_pattern(text_pattern)
    logo_layer.set_preserve_trans(true)
    bucket_fill(logo_layer, PATTERN_BUCKET_FILL, NORMAL_MODE,
		100, 0, false, 0, 0)
    selection_none(img)
    edit_clear(text_shadow_layer)
    edit_clear(drop_shadow_layer)
    palette_set_background(bg_color)
    shadow_layer.fill(BG_IMAGE_FILL)
    rect_select(img, b_size_2, b_size_2, width - b_size, height - b_size,
		REPLACE, true, b_size_2)
    palette_set_background([0, 0, 0])
    edit_fill(shadow_layer, BG_IMAGE_FILL)
    selection_layer_alpha(logo_layer)
    img.add_layer_mask(text_shadow_layer, tsl_layer_mask)
    palette_set_background([255, 255, 255])
    edit_fill(tsl_layer_mask, BG_IMAGE_FILL)
    selection_feather(img, f_size)
    palette_set_background([63, 63, 63])
    edit_fill(drop_shadow_layer, BG_IMAGE_FILL)
    palette_set_background([0, 0, 0])
    edit_fill(text_shadow_layer, BG_IMAGE_FILL)
    palette_set_foreground([255, 255, 255])
    blend(text_shadow_layer, FG_BG_RGB, NORMAL_MODE, SHAPEBURST_ANGULAR,
	  100, 0, REPEAT_NONE, false, 0, 0, 0, 0, 1, 1)
    selection_none(img)
    palette_set_foreground(blend_fg)
    palette_set_background(blend_bg)
    blend(blend_layer, FG_BG_RGB, NORMAL_MODE, LINEAR, 100, 0, REPEAT_NONE,
	  false, 0, 0, 0, 0, width, 0)
    PDB.plug_in_mosaic(RUN_NONINTERACTIVE, img, blend_layer, 12, 1, 1, 0.7,
		       135, 0.2, true, false, tile_type, 1, 0)
    logo_layer.translate(- b_size_2, - b_size_2)
    blend_layer.translate(- b_size, - b_size)
    text_shadow_layer.translate(- ts_size, - ts_size)
    drop_shadow_layer.translate(ds_size, ds_size)
    selection_layer_alpha(blend_layer)
    img.add_layer_mask(drop_shadow_layer, dsl_layer_mask)
    palette_set_background([255, 255, 255])
    edit_fill(dsl_layer_mask, BG_IMAGE_FILL)
    img.remove_layer_mask(drop_shadow_layer, APPLY)
    selection_none(img)
  ensure
    patterns_set_pattern(old_pattern)
    palette_set_foreground(old_fg)
    palette_set_background(old_bg)
  end
end


def textured_logo_alpha(img, logo_layer,
			b_size,
			text_pattern,
			tile_type,
			bg_color,
			blend_fg,
			blend_bg)
  img.undo_group {
    apply_textured_logo_effect(img, logo_layer, b_size, text_pattern,
			       tile_type, bg_color, blend_fg, blend_bg)
  }
  displays_flush
end


register("ruby_fu_textured_logo_alpha",
	 "Creates textured logos with blended backgrounds, highlights, and shadows",
	 "Creates textured logos with blended backgrounds, highlights, and shadows",
	 "Spencer Kimball",
	 "Spencer Kimball",
	 "1996",
	 "<Image>/Ruby-Fu/Alpha to Logo/Textured...",
	 "RGBA",
	 [
	   [RF_IMAGE,      "image",       "Image"],
	   [RF_DRAWABLE,   "drawable",    "Drawable"],
	   [RF_ADJUSTMENT, "border_size", "Border Size (pixels)", 20,
	     [1, 100, 1, 10, 0, 1]],
	   [RF_PATTERN,    "pattern",     "Pattern", "Fibers"],
	   [RF_OPTION,     "type",        "Mosaic Tile Type", 0,
	     ["Squares", "Hexagons", "Octagons"]],
	   [RF_COLOR,      "bg_color",       "Background Color",
	     Color::new(255, 255, 255)],
	   [RF_COLOR,      "starting_blend", "Stating Blend",
	     Color::new(32, 106, 0)],
	   [RF_COLOR,      "ending_blend",   "Ending Blend",
	     Color::new(0, 0, 106)]
	 ]
) { |*args|
  textured_logo_alpha(*args)
}


def textured_logo(text, size, fontname, text_pattern, tile_type, bg_color,
		  blend_fg, blend_bg)
  img    = Image::new(256, 256, RGB)
  b_size = scale(size, 0.1)
  text_layer = text_fontname(img, -1, 0, 0, text, b_size, true, size, PIXELS,
			     fontname)
  img.undo_disable
  text_layer.set_name(text)
  apply_textured_logo_effec(img, text_layer, b_size, text_pattern, tile_type,
			    bg_color, blend_fg, blend_bg)
  img.undo_enable
  display_new(img)
end


register("ruby_fu_textured_logo",
	 "Creates textured logos with blended backgrounds, highlights, and shadows",
	 "Creates textured logos with blended backgrounds, highlights, and shadows",
	 "Spencer Kimball",
	 "Spencer Kimball",
	 "1996",
	 "<Toolbox>/Xtns/Ruby-Fu/Logos/Textured...",
	 "RGBA",
	 [
	   [RF_STRING,     "text",        "Text", "The GIMP"],
	   [RF_ADJUSTMENT, "font_size",   "Font Size (pixels)", 200,
	     [1, 1000, 1, 10, 0, 1]],
	   [RF_FONT,       "font",        "Font", "-*-cuneifontlight-*-r-*-*-24-*-*-*-p-*-*-*"],
	   [RF_PATTERN,    "pattern",     "Pattern", "Fibers"],
	   [RF_OPTION,     "type",         "Mosaic Tile Type", 0,
	     ["Squares", "Hexagons", "Octagons"]],
	   [RF_COLOR,      "bg_color",     "Background Color",
	     Color::new(255, 255, 255)],
	   [RF_COLOR,      "starting_blend", "Stating Blend",
	     Color::new(32, 106, 0)],
	   [RF_COLOR,      "ending_blend", "Ending Blend",
	     Color::new(0, 0, 106)]
       ]
) { |*args|
  textured_logo(*args)
}


main()
